// use twelite mwx c++ template library
#include <TWELITE>
#include <NWK_SIMPLE>
#include <BRD_APPTWELITE>
#include "Common/i2c.hpp"
#include "Common/bme280.hpp"

I2c g_i2c;
Bme280 g_bme280;
BME280_DATA g_bme280Data;

/*** Config part */
// application ID
const uint32_t APP_ID = 0x1234abcd;
//const uint32_t APP_ID = 0x67720120;

// channel
const uint8_t CHANNEL = 13;

/*** function prototype */
MWX_APIRET transmit();
void receive();

/*** application defs */
const char APP_FOURCHAR[] = "BAT1";
uint8_t u8devid = 0;

uint16_t au16AI[5];
uint8_t u8DI_BM;

/*** setup procedure (run once at cold boot) */
void setup() {
	g_i2c.initBus();
	g_bme280.init(g_i2c);
	// init vars
	for(auto&& x : au16AI) x = 0xFFFF;
	u8DI_BM = 0xFF;

	/*** SETUP section */
	auto&& brd = the_twelite.board.use<BRD_APPTWELITE>();

	// check DIP sw settings
	u8devid = (brd.get_M1()) ? 0x00 : 0xFE;

	// the twelite main class
	the_twelite
		<< TWENET::appid(APP_ID)    // set application ID (identify network group)
		<< TWENET::channel(CHANNEL) // set channel (pysical channel)
		<< TWENET::rx_when_idle();  // open receive circuit (if not set, it can't listen packts from others)

	// Register Network
	auto&& nwksmpl = the_twelite.network.use<NWK_SIMPLE>();
	nwksmpl << NWK_SIMPLE::logical_id(u8devid); // set Logical ID. (0x00 means parent device)

	// Timer setup
	Timer0.begin(32, true); // 32hz timer


	the_twelite.begin(); // start twelite!

	/*** INIT message */
	Serial << "--- BRD_APPTWELITE(" << int(u8devid) << ") ---" << '\n';
}

/*** loop procedure (called every event) */
void loop() {


	if (Timer0.available()) {
		static uint8_t u16ct;
		u16ct++;

		if (u8devid == 0xfe) { // finished the first capture
			if ((u16ct % 32) == 0) { // every 32ticks of Timer0
				g_bme280.getData(g_i2c, &g_bme280Data);
				transmit();
			}
		}
	}
}

/*** transmit a packet */
MWX_APIRET transmit() {
	if (auto&& pkt = the_twelite.network.use<NWK_SIMPLE>().prepare_tx_packet()) {
		// set tx packet behavior
		pkt << tx_addr(u8devid == 0 ? 0xFE : 0x00)  // 0..0xFF (LID 0:parent, FE:child w/ no id, FF:LID broad cast), 0x8XXXXXXX (long address)
			<< tx_retry(0x1) // set retry (0x1 send two times in total)
			<< tx_packet_delay(0,50,10); // send packet w/ delay (send first packet with randomized delay from 100 to 200ms, repeat every 20ms)

		// prepare packet payload
		pack_bytes(pkt.get_payload() // set payload data objects.
			, make_pair(APP_FOURCHAR, 4) // string should be paired with length explicitly.
			, uint8_t(u8DI_BM)
		);

		pack_bytes(pkt.get_payload(), uint16_t(g_bme280Data.temp));
		pack_bytes(pkt.get_payload(), uint16_t(g_bme280Data.humi));
		pack_bytes(pkt.get_payload(), uint16_t(g_bme280Data.press));
		
		// do transmit 
		return pkt.transmit();
	}
	return MWX_APIRET(false, 0);
}


/* Copyright (C) 2019 Mono Wireless Inc. All Rights Reserved.    *
 * Released under MW-SLA-*J,*E (MONO WIRELESS SOFTWARE LICENSE   *
 * AGREEMENT).                                                   */